#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <map>

using namespace std;

enum COLOR {
    red = 1, green, blue, cyan, magenta, yellow, black, white, c_undef = 10000
};

enum SIZE {
    s0 = 0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s_undef = 10000 
};

enum TAGS {
    nothing = 0, 
    MIN_TAG,
    b_tag, em_tag, i_tag, pl_tag, s_tag, tt_tag, u_tag, size_tag, color_tag,
    MAX_TAG
};

enum NODETYPE {
    TextNodeType, ContextNodeType
};

enum EM {
    TRUE = true, FALSE = false, DONT_CARE = 10000
};

bool is_whitespace(char c)
{
   return (c == 0x20 || c == 0x0D || c == 0x0A || c == 0x09);
};

/* Implement node type */
class Node {
    public:
	Node() {};
	~Node() {};

	virtual NODETYPE getNodeType() = 0;
	virtual void emit() = 0;
};

/* Implement text node */
class TextNode : public Node{
    public:
	TextNode() {};
	~TextNode() {};
	TextNode(const string &str) {
	    setText(str);
	}

	void setText(const string &str) { 
	    s.assign(str); 
	}
	string getText() const { return s;}

	virtual NODETYPE getNodeType() { return TextNodeType; }
	virtual void emit() {
	    cout << s;
	}

    private:
	string s;
};

/* Implement n-ary tree and nodes*/
template <class T>
class Tree {
    public:
	Tree() { nodes = new vector<T *>; };
	~Tree() { 
	    for(int i=0; i < nodes->size(); i++) {
		delete (*nodes)[i];
	    }
	    delete nodes; 
	};
	
	void addNode(T *t) {
	    nodes->push_back(t);
	}

	T* getNode(int i) {
	    assert(nodes->size() > i);
	    return &((*nodes)[i]);
	}
	
	void removeNode(int i) {
	    assert(nodes->size() > i);
	    vector <T *> :: iterator it = nodes->begin() + i;
	    nodes->erase(it);
	}

	T *getAndRemoveNode(int i) {
	    T *t = getNode(i);
	    removeNode(i);
	    return t;
	}

	int getSize() {
	    return nodes->size();
	}

    protected:
	vector <T *> *nodes;
};

class ContextNode;
class TextEntry {
    public:
	TextEntry(const string &s, ContextNode *c);
	~TextEntry();
	void setText(const string &s);
	void setContext(ContextNode *c);
	void show() ;
	string getText() const;

	int get_u() const;
	bool get_b() const;
	EM get_em() const;
	bool get_i() const;
	bool get_s() const;
	bool get_tt() const;
	COLOR get_color() const;
	SIZE get_size() const;

	bool isOnlySpace();
	bool isBeginBySpace();
	bool isEndBySpace();
	void TrimBeginSpace();
	void TrimEndSpaceButOne();
	bool isEmpty();

	bool isUsed(TAGS t); 

    private:
	    string str;
	    int u;
	    bool b;
	    EM em;
	    bool i;
	    bool s;
	    bool tt;
	    COLOR color;
	    SIZE size;
};

typedef vector<TextEntry> TextEntries;
TextEntries *globalTextEntries;

typedef pair <int, int> KEY_TYPE;
typedef vector<int> VALUE_TYPE;
class STAT_TYPE: public map<KEY_TYPE, VALUE_TYPE> 
{
/*
    public:
	VALUE_TYPE get_stat(const KEY_TYPE & k) const {
	    const_iterator it = find(k);
	    if (it == end()) {
		//calculate
		int count = 0;
		VALUE_TYPE v;
		for (int j=0; j < MAX_TAG; j++)
		    v.push_back(0);

		TextEntries::iterator text_it;
		for (int i=k.first; i < k.second; i++) {
		    for (int j=MIN_TAG+1; j < MAX_TAG; j++) {
			if((*globalTextEntries)[i].isUsed((TAGS)j)) v[j]++;
		    }
		}
		operator[](k) = v;
		return v;
	    } else {
		return (*it).second;
	    }
	}
	*/
};

STAT_TYPE tag_stat;

class ContextNode : public Tree <Node>, public Node
{
    public:
	ContextNode() : u(false), b(false), em(FALSE), i(false), s(false),
	    tt(false), color(c_undef), size(s_undef), parent(NULL), 
	    modified(nothing){
	}

	ContextNode(ContextNode *c) {
	    u = c->get_u();
	    b = c->get_b();
	    em = c->get_em();
	    i = c->get_i();
	    s = c->get_s();
	    tt = c->get_tt();
	    color = c->get_color();
	    size = c->get_size();
	    parent = c;
	    modified = nothing;
	};
	~ContextNode() {};

	virtual NODETYPE getNodeType() { return ContextNodeType; }

	/* accessor */
	int get_u() { return u; }
	bool get_b() { return b; }
	EM get_em() { return em; }
	bool get_i() { return i; }
	bool get_s() { return s; }
	bool get_tt() { return tt; }
	COLOR get_color() { return color; }
	SIZE get_size() { return size;}

	void do_b() {b = true; modified = b_tag;}
	void do_em() { 
	    if (s) em = DONT_CARE; 
	    else switch(em) {
		case TRUE: em = FALSE; break; 
		case FALSE: em = TRUE; break;
	    }
	    modified = em_tag;}
	void do_i() {i = true; modified = i_tag;}
	void do_pl() {
	    s = i = b = tt = false; 
	    u = 0; 
	    em = FALSE;
	    modified = pl_tag;}
	void do_s() {s = true; em = DONT_CARE; modified = s_tag;}
	void do_tt() { tt = true; modified = tt_tag;}
	void do_u() { if (u < 3) u++; modified = u_tag;};
	void do_size(int i) { size = (SIZE) i; modified = size_tag;};
	void do_size(SIZE i) { size = i; modified = size_tag;} 
	void do_color(COLOR c) { color = c; modified = color_tag;}

	ContextNode * getParent() { return parent; }

	void addTextNode(TextNode *t) {
	    if (t->getText().length() == 0) {
		delete t;
	    } else {
		nodes->push_back(static_cast<Node *>(t));
	    }
	}

	int findBoundary(int current_row)
	{
	    int num = calcContextMatch(&(*globalTextEntries)[current_row]);
	    TextEntries::iterator it = globalTextEntries->begin() + current_row;
	    int i = 1;
	    for(; it != globalTextEntries->end(); it++) {
		if (num > calcContextMatch(it)) break;
		i++;
	    }

	    return i;
	}
	
	void addTextEntry(TextEntry *t, int current_row) {
	    int num = 0;
	    ContextNode *max_node = findMaxMatchNode(num, this, t);
	    ContextNode *curNode = max_node;
	    ContextNode *tmpNode = NULL;
	    bool isOnlySpace = t->isOnlySpace();
	    
	    if (curNode->get_size() != t->get_size()) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_size(tmpNode->get_size());
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if (!(isOnlySpace && (t->get_u() == 0))) { 
		if (curNode->get_color() != t->get_color()) {
		    tmpNode = new ContextNode(curNode);
		    tmpNode->do_color(t->get_color());
		    curNode->addNode(tmpNode);
		    curNode = tmpNode;
		}
	    }

	    if (curNode->get_u() != t->get_u()) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_u();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if ((curNode->get_b() != t->get_b()) && !isOnlySpace) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_b();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if ((curNode->get_em() != t->get_em()) && !isOnlySpace) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_em();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if ((curNode->get_i() != t->get_i()) && !isOnlySpace) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_i();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if ((curNode->get_s() != t->get_s()) && !isOnlySpace) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_s();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }

	    if (curNode->get_tt() != t->get_tt()) { 
		tmpNode = new ContextNode(curNode);
		tmpNode->do_tt();
		curNode->addNode(tmpNode);
		curNode = tmpNode;
	    }
	    curNode->addTextNode(new TextNode(t->getText()));
	}

	ContextNode * findMaxMatchNode(int &max, ContextNode* node_has_max, 
		TextEntry *t) {

	    // Do a DFS and find the maximum number of context matches
	    // Note: only the rightest child should be examined or else
	    // the order will be xxx
	    ContextNode * node_max;

	    int num_match = calcContextMatch(t);
	    if (num_match >= max) {
		node_max = this;
		max = num_match;
	    } else {
		node_max = node_has_max;
	    }

	    if (nodes->size() > 0) {
		vector <Node *> :: reverse_iterator it(nodes->end());
		if ( (*it)->getNodeType() == ContextNodeType) {
		    node_max = 
			dynamic_cast<ContextNode *>(*it)->findMaxMatchNode(max, 
			    node_max, t);
		}
	    }
	    
	    return node_max;
	}

	int calcContextMatch(TextEntry *t) {
	    int num = 0;
	    if (get_u() || !t->get_u()) num++;
	    if (get_b() || !t->get_b()) num++;
	    if (get_i() || !t->get_i()) num++;
	    if (get_s() || !t->get_s()) num++;
	    if (get_tt() || !t->get_tt()) num++;
	    if ((t->get_em() == DONT_CARE) || (get_em() == FALSE )) num++;
	    if (get_color() == t->get_color()) num++;
	    if (get_size() == t->get_size()) num++;

	    if (get_u() && !t->get_u()) num = -1;
	    if (get_b() && !t->get_b()) num = -1;
	    if (get_i() && !t->get_i()) num = -1;
	    if (get_s() && !t->get_s()) num = -1;
	    if (get_tt() && !t->get_tt()) num = -1;
	    return num;
	}

	void emit() {
	    string str;
	    switch (modified) {
		case nothing: break;
		case b_tag: str = "B"; 
			    break;
		case em_tag: if ((s == s_undef) && (em == TRUE))
				 str = "EM"; 
			     else
				 modified = nothing;
			     break;
		case i_tag: str = "I";
			    break;
		case s_tag: str = "S";
			    break;
		case pl_tag: str = "PL";
			     break;
		case tt_tag: str = "TT";
			     break;
		case u_tag: str = "U";
			    break;
		case size_tag: str = (char)(size + '0');
			       break;
		case color_tag: switch(color) {
				    case red: str = "r"; break;
				    case green: str = "g"; break;
				    case blue: str = "b"; break;
				    case cyan: str = "c"; break;
				    case magenta: str = "m"; break;
				    case yellow: str = "y"; break;
				    case black: str = "k"; break;
				    case white: str = "w"; break;
				}
				break;
		default: cerr << "Unrecognized modified tag: " << modified;
			 exit(-1);
			 break;
	    }
	    if (modified == nothing) {
		emitChildren();
	    } else {
		cout << "<" << str << ">";
		emitChildren();
		cout << "</" << str << ">";
	    }
	}

	void emitChildren()
	{
	    for (int i=0; i < nodes->size(); i++) {
		nodes->operator[](i)->emit();
	    }
	}

    private:
	ContextNode *parent;
	TAGS modified;
	int u;
	bool b;
	EM em;
	bool i;
	bool s;
	bool tt;
	COLOR color;
	SIZE size;
};

/* Implement text entries */
TextEntry::TextEntry(const string &s, ContextNode *c) {
    str.assign(s);
    setContext(c);
}
TextEntry::~TextEntry() {};

void TextEntry::setText(const string &s) {
    str.assign(s);
}

void TextEntry::setContext(ContextNode *c) {
    u = c->get_u();
    b = c->get_b();
    em = c->get_em();
    i = c->get_i();
    s = c->get_s();
    tt = c->get_tt();
    color = c->get_color();
    size = c->get_size();
}

void TextEntry::show() {
    cout << "U:" << u << " " << ((b) ? "B " : "")
	<< ((em == TRUE) ? "EM " : ((em==DONT_CARE) ? "EM dont care " : "")) << ((i) ? "I " : "")
	<< ((s) ? "S " : "") << ((color == c_undef) ? "" : "C ")
	<< ((size == s_undef) ? "" : "S ") 
	<< ((tt) ? "TT " : "") << "| " << "\"" << str << "\"" << endl;
};

string TextEntry::getText() const { return str; }
int TextEntry::get_u() const { return u; }
bool TextEntry::get_b() const { return b; }
EM TextEntry::get_em() const { return em;}
bool TextEntry::get_i() const { return i; }
bool TextEntry::get_s() const { return s; }
bool TextEntry::get_tt() const { return tt; }
COLOR TextEntry::get_color() const { return color; }
SIZE TextEntry::get_size() const { return size; }

bool TextEntry::isOnlySpace() {
    return ((str.length() == 0) || 
	    str.find_first_not_of(' ') == string::npos);
}

bool TextEntry::isBeginBySpace() {
    return (is_whitespace(str[0]));
}

bool TextEntry::isEmpty()
{
    return (str.length() == 0);
}

bool TextEntry::isEndBySpace() {
    int tmp = str.length();
    return is_whitespace(str[tmp-1]);
}

void TextEntry::TrimBeginSpace() {/*
    int i = str.find_first_not_of(' ');
    if ( i != string::npos) str.erase(0, i); */
}

void TextEntry::TrimEndSpaceButOne() {
    int i = str.find_last_not_of(' ');
    if (str.length() > i + 1) str.erase(i + 1); 
}

bool TextEntry::isUsed(TAGS i)
{
    switch(i) {
	case b_tag: return get_b();
	case em_tag: return (get_em()==TRUE);
	case i_tag: return get_i();
	case s_tag: return get_s();
	case tt_tag: return get_tt();
	case u_tag: return (get_u() > 0);
	case size_tag: return (get_size() != s_undef);
	case color_tag: return (get_color() != c_undef);
    }
    return false;
}

TextEntries *parse() {
    char c;

    string str;
    ContextNode *root = new ContextNode();
    ContextNode *currentNode = root;
    ContextNode *newNode = NULL;
    TextNode *textNode = NULL;
    TextEntries *entries = new TextEntries;

    while (!cin.get(c).eof()) {
	switch(c) {
	    case '<': // tags
		cin.get(c);
		if (c == '/') {
		    currentNode = currentNode->getParent();
		    currentNode->removeNode(0);
		    while (!cin.eof() && c != '>') {
			cin.get(c);
		    }
		} else {
		    str = "";
		    cin.unget();
		    newNode = new ContextNode(currentNode);

		    while(!cin.eof()) {
			cin >> c;
			if (c == '>') break;
			str += c;
		    }
		    if (str == "B") { newNode->do_b(); }
		    else if (str == "EM") { newNode->do_em(); }
		    else if (str == "I") { newNode->do_i(); }
		    else if (str == "PL") { newNode->do_pl(); }
		    else if (str == "S") { newNode->do_s(); }
		    else if (str == "TT") { newNode->do_tt(); }
		    else if (str == "U") { newNode->do_u(); }
		    else {
			if (str.length()>1) {
			    cerr << "Too long: " << str << endl;
			    exit(-1);
			}
			if (str[0] >= '0' && str[0] <= '9') {
			    newNode->do_size(str[0] - '0');
			} else {
			    switch(str[0]) {
				case 'r': 
				    newNode->do_color(red); break;
				case 'g':
				    newNode->do_color(green); break;
				case 'b':
				    newNode->do_color(blue); break;
				case 'c':
				    newNode->do_color(cyan); break;
				case 'm':
				    newNode->do_color(magenta); break;
				case 'y':
				    newNode->do_color(yellow); break;
				case 'w':
				    newNode->do_color(white); break;
				case 'k':
				    newNode->do_color(black); break;
				default:
				    cerr << "Unrecognized tag";
				    exit(-1);
			    }
			}
		    };
		    currentNode->addNode(newNode); 
		    currentNode = newNode; 
		} 
		break;

	    case 0x0D:
	    case 0x0A:
	    case 0x20:
	    case 0x09:
		// whitespace
		str.assign(" ");
		while (!cin.eof()) {
		    cin.get(c);
		    if (!is_whitespace(c)) {
			cin.unget();
			break;
		    }
		}
		entries->push_back(TextEntry(str, currentNode));
		break;
		

	    default:
		cin.unget();
		str = "";
		while (!cin.eof()) {
		    cin.get(c);
		    if (is_whitespace(c)) {
			cin.unget();
			break;
		    } else if (c == '<') {
			cin.unget();
			break;
		    } else {
			str += c;
		    }
		}

		entries->push_back(TextEntry(str, currentNode));
		break;
	};
    }
    
    delete root;
    return entries;
}

int main(int, char**)
{
    TextEntries *t = parse();
    globalTextEntries = t;

    // Remove junk entries
    TextEntries::iterator it = t->begin();
    TextEntries::iterator last_it = NULL;

    for (; it != t->end(); it++) {
	if (last_it != NULL && last_it->isEndBySpace() && 
		it->isBeginBySpace()) {
	    it->TrimBeginSpace();
	    if (!last_it->get_tt()) last_it->TrimEndSpaceButOne();
	}
	last_it = it;
    }
    t->erase(last_it);

    for (it=t->begin(); it != t->end(); it++) {
	if (it->isEmpty()) t->erase(it);
    }

    // Construct tree
    ContextNode *root = new ContextNode();
    int rownum = 0;
    for (it=t->begin(); it != t->end(); it++) {
	root->addTextEntry(it, rownum++);
    }

    
    root->emit();
    //cout << "END" << endl;

    delete t;
}
